<?php
/**
 * Created by PhpStorm.
 * User: rafaelporto
 * Date: 23/10/2015
 * Time: 10:20
 */

namespace Service\Calendar;

use Doctrine\ORM\EntityManager as EntityManager;
use \Service\Structure as Structure;
use Service\CalendarDetail as CalendarDetail;
use Util\Utilities;

class ResourceCalendar {

    CONST HOUR_DAY = 28800;
    CONST END_WEEK_HOUR_DAY = 0;

    /** @var EntityManager $entityManager */
    private $entityManager;

    /** @var \Service\Structure $structureService */
    private $structureService;
    /**
     * @var Utilities
     */
    private $utilities;
    /**
     * @var \Service\Calendar
     */
    private $calendarService;

    public function __construct(EntityManager $entityManager, Structure $structureService,\Service\Calendar $calendarService,
                                Utilities $utilities){
        $this->entityManager = $entityManager;
        $this->structureService = $structureService;
        $this->utilities = $utilities;
        $this->calendarService = $calendarService;
    }

    /**
     * This method return a list of calendars associated with a resource.
     * Should return ordered by relation distance with the resource.
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource
     * @return \Zeedhi\DevOps\Model\Projectmanagement\Entities\Calendar[]
     */
    private function getCalendars($resource) {
        $calendars = array();

        $resourceCalendar = $resource->getCalendar();
        array_push($calendars, $resourceCalendar);

        $structure = $this->structureService->getStructureByResource($resource);
        if(!empty($structure)){
            $structureCalendar = $structure->getAllCalendar();
            $calendars = array_merge($calendars, $structureCalendar);
        }

        return $calendars;
    }

    /**
     * This method considers that the new day has a distant relationship with the resource.
     * This want mean that has a tendency to preserve the day that is already set.
     * @param $day
     * @param $oldDay
     * @return null|$day|$oldDay
     */
    private function getDayHours($day, $oldDay) {
        $dayToReturn = null;

        if($day['TYPE'] == CalendarDetail::OUT_DAY_TYPE)
            $day['DAILY_HOURS'] = '0';

        if(is_null($oldDay)){
            $dayToReturn = $day;
        } else if($oldDay['IS_EXCEPTION'] == CalendarDetail::EXCEPTION_TRUE) {
            $dayToReturn = $oldDay;
        } else if($day['IS_EXCEPTION'] == CalendarDetail::EXCEPTION_TRUE){
            $dayToReturn = $day;
        } else {
            $dayToReturn = $oldDay;
        }

        return $dayToReturn;
    }

    /**
     * Return associative array indexed by dates and with daily hours as values.
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource
     * @param \DateTime $beginDate
     * @param \DateTime $endDate
     * @return array( date => hour)
     */
    private function getWorkingDays($resource, $beginDate, $endDate) {
        $workingDays = array();
//        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\CalendarDates $calendarDates */
//        $calendarDates = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\CalendarDates');
//
//        $calendars = $this->getCalendars($resource);
//
//        foreach($calendars as $calendar) {
//            if(!empty($calendar)){
//                $calendarWorkingDays = $calendarDates->getExceptionCalendarDates($calendar->getId(), $beginDate, $endDate);
//                foreach($calendarWorkingDays as $workingDay){
//                    $workingDayData = empty($workingDays[$workingDay['DATA']]) ? null : $workingDays[$workingDay['DATA']];
//                    $workingDays[$workingDay['DATA']] = $this->getDayHours($workingDay, $workingDayData);
//                }
//            }
//        }

        $resourceCalendar = $resource->getCalendar();
        $structure =$this->structureService->getStructureByResource($resource);
        $structureCalendar = $this->utilities->getNextCalendar($structure);

        if($structureCalendar){
            $structureDates = $this->calendarService->getCalendarDates($structureCalendar, $beginDate, $endDate);
        }else{
            $structureDates = array();
        }
        if($resourceCalendar && $resourceCalendar->getCalendarDefault()){
            $resourceDates = $this->calendarService->getCalendarDates($resourceCalendar, $beginDate, $endDate);
        }else{
            $resourceDates = array();
        }
        $calendarDates = $this->calendarService->mergeCalendarDates($resourceDates,$structureDates);

        foreach($calendarDates as $workingDay){
            $workingDays[$workingDay['DATA']] = intval($workingDay['DAILY_HOURS']);
        }
        return $workingDays;
    }

    /**
     * Return associative array indexed by dates and with daily hours as values but consider default hour as self::HOUR.
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource
     * @param $beginDate
     * @param $endDate
     * @return array( date => hour)
     * @throws \Exception
     */
    public function getDatesCalendar($resource, $beginDate, $endDate) {

        $begin = \DateTime::createFromFormat('d/m/Y', $beginDate);
        $end = \DateTime::createFromFormat('d/m/Y', $endDate);

        if(!$begin || !$end){
            throw new \Exception("data do período não foi informada.");
        }

        $end = $end->modify('+1 day');

        $interval = \DateInterval::createFromDateString('1 day');
        $dateRange = new \DatePeriod($begin, $interval, $end);

        $dates = array();
        foreach($dateRange as $date){
            if(in_array($date->format('N'),array('6','7'))){
                $dates[$date->format('d/m/Y')] = self::END_WEEK_HOUR_DAY;
            }else{
                $dates[$date->format('d/m/Y')] = self::HOUR_DAY;
            }
        }
        $workingDays = $this->getWorkingDays($resource, $beginDate, $endDate);
        foreach($workingDays as $date => $hour){
            $dates[$date] = $hour;
        }

        return $dates;
    }
} 